#pragma once
#include <memory>
#include <mutex>

namespace eda {
typedef void (*IoTrigger)(int level);
typedef void (*serialcallback)(char *buffer);
typedef void (*HardwareTrigger)();

class EdaIo{
public:
    static EdaIo* getInstance();
    static void close_io();
    ~EdaIo();
    /**
     * @brief 打开激光
     * 
     */
    void openLaser();
    /**
     * @brief 关闭激光
     * 
     */
    void closeLaser();
    /**
     * @brief 设置状态指示灯
     * 
     * @param good 
     */
    void setScanStat(bool good);
    /**
     * @brief 打开alarm 指示灯
     * 
     */
    void openAlarm();
    /**
     * @brief 关闭alarm 指示灯
     * 
     */
    void closeAlarm();
    /**
     * @brief
     *
     * @param section 1~4
     * @return int
     */
    int enableLightSection(int section);
    /**
     * @brief
     *
     * @param section 1~4
     * @return int
     */
    int disableLightSection(int section);
    /**
     * @brief 设置output1 输出 [高/低]
     * 
     * @param high 
     */
    void setDo1High(bool high);
    /**
     * @brief 设置output2 输出 [高/低]
     * 
     * @param high 
     */
    void setDo2High(bool high);

    // void setAimerColor(RGBColor color);

    /**
     * @brief 注册input触发回调函数
     * 
     * @param callback 
     */
    void registerInput(IoTrigger callback);
    /**
     * @brief 注册register按键 回调函数
     * 
     * @param callback 
     */
    void registerTrigger(IoTrigger callback);
    /**
     * @brief 注册Tune按键 回调函数
     * 
     * @param callback 
     */
    void registerTune(IoTrigger callback);
    /**
     * @brief 注册硬件触发
     * 
     * @param callback 
     */
    void registerHardware(HardwareTrigger callback);

    /**
     * @brief set RGB light
     *
     * @param light 0: Close; 1: Red; 2: Green; 3: Blue,
     * @return int
     */
    void setRgbLight(uint8_t light);

    /**
     * @brief Set the Work Mode object.
     * @param mode  work mode:
     *     -0x00 连续触发模式.
     *     -0x01 软件触发模式.
     *     -0x02 外部IO触发，上升沿触发.
     *     -0x03 外部IO触发，下降沿触发.
     *     -0x04 外部IO Level触发.
     */
    int setWorkMode(int mode);
#ifdef SUPPORT_LIGHT_CUSTOM
    /**
     * @brief 自定义光源模式, 手动打开光源。
     *          会改变工作模式，自动设置为：5
     * @param timeout 最长打开光源时间毫秒，最大3秒。超过自动关闭。
     * @return int 
     */
    int open_light();
    /**
     * @brief 自定义光源模式, 手动关闭光源。
     * 
     * @return int 
     */
    int close_light();
#endif
    /**
     * @brief 初始化IO 设置
     * 
     */
    void setup();
    /**
     * @brief call light flsh
     * 
     */
#ifdef ENABLE_LIGHT_TIGGER
    void callLightFlash();
    void setLightFlashCount(int count);
    void setLightFlashTime(int ms);
#endif
#ifdef ENABLE_LIQUID
    int getLiquid();
    /**
     * @brief Set the Liquid object
     * 
     * @param value 0~46000
     * @return int 
     */
    int setLiquid(int value);
#endif
};

}
