#include "eda/CameraManger.h"
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <pthread.h>
#include <string.h>
#include <iostream>
#include "eda/eda-io.h"
#include "eda/camera_2311.h"
// #include <mutex>

int M_APP_STOP = 0;
int M_WIDTH;
int M_HEIGHT;
char *M_IMAGE_BUFF;
int M_CAPTURE=0;
int M_MOD = 1;
int M_SIZE=0;
int M_RUNNING=1;
// std::mutex M_LOCK;

void my_handler(int sig){
    if(sig == SIGINT){
        printf("Ctrl + c pressed!\n");
        M_APP_STOP = 1;
    }
}

extern "C"{
    void savejpg(char * pdata, char *jpg, int width, int height, char quality );
}

void image_handler(){
    int i = 0;
    char output[20];
    while (M_RUNNING)
    {
        if(M_APP_STOP == 1){
            break;
        }
        if(M_MOD == 0){
            if(M_CAPTURE == 1){
                M_CAPTURE = 0;
                
                if(i%100 == 50){
                    sprintf(output,"output_%d.jpg",i);
                    savejpg(M_IMAGE_BUFF, output, M_WIDTH, M_HEIGHT, 100);
                    printf("[Test] continue capture %s\n",output);
                    memset(output,0,sizeof(output));
                    memset(M_IMAGE_BUFF,0,sizeof(M_SIZE));
                    M_SIZE = 0;
                }
                i++;
                // TODO do something
            }
        }else{
            if(M_CAPTURE == 1){
                // M_LOCK.lock();
                M_CAPTURE = 0;
                sprintf(output,"output_%d.jpg",i++);
                savejpg(M_IMAGE_BUFF, output, M_WIDTH, M_HEIGHT, 100);
                printf("[Test] trigger capture %s\n",output);
                memset(output,0,sizeof(output));
                memset(M_IMAGE_BUFF,0,sizeof(M_SIZE));
                M_SIZE = 0;
                // M_LOCK.unlock();
            }
        }
        usleep(50);
    }
}

/**
 * @brief 注册函数，不要执行太多动作，只拷贝buf
 * 
 * @param img_buff 
 * @param img_len 
 * @return int 
 */
int image_callback(char *img_buff, int img_len){
    // M_LOCK.lock();
    if(M_IMAGE_BUFF){
        // printf("Image callback len: %d\n");
        memcpy(M_IMAGE_BUFF, img_buff, img_len);
        M_SIZE = img_len;
        M_CAPTURE = 1;
    }
    // M_LOCK.unlock();
    return 0;
}

int main(int argc, char *argv[]){
    int ret;
    pthread_t t_image_thread;

    struct sigaction sigIntHandler;
    sigIntHandler.sa_handler = my_handler;
    sigemptyset(&sigIntHandler.sa_mask);
    sigIntHandler.sa_flags = 0;
    sigaction(SIGINT, &sigIntHandler, NULL);
    
    eda::Camera *t_camera = eda::load_default();
    eda::EdaIo *em = eda::EdaIo::getInstance();
    em->setup();
    em->enableLightSection(1);
    em->enableLightSection(2);
    em->enableLightSection(3);
    em->enableLightSection(4);
    em->openLaser();
    if(t_camera){
        t_camera->callback_image_ready(image_callback);
        int mod;
        int width;
        int height;
        printf("[Input] work mode[0: CONTINUE, 1: HARDWARE, 4: LEVEL, 5: SOFTWARE]: ");
        std::cin >> mod;
        printf("[Input] width: ");
        std::cin >> width;
        printf("[Input] height: ");
        std::cin >> height;
        ret = t_camera->open(mod, width, height);
        printf("[Debug] Camera open: %d\n", ret);
        if(ret != 0){
            return;
        }

        printf("[Input] gain: ");
        int gain;
        std::cin >> gain;
        t_camera->set_gain(gain);
        printf("[Input] exposure: ");
        int exposure;
        std::cin >> exposure;
        t_camera->set_exposure(exposure);
        M_MOD = mod;
        M_WIDTH = width;
        M_HEIGHT = height;
        printf("[Debug] mod: %d\n",M_MOD);
        M_IMAGE_BUFF = (char*) malloc((width*height)*sizeof(char));
        if(ret == 0){
            pthread_create(&t_image_thread, NULL, image_handler, NULL);
            if(mod == 5){
                if(t_camera->name() == eda::CameraName::OV2311){
                    eda::Camera_2311 *o_camera = (eda::Camera_2311*)t_camera;
                    int i_c;
                    while(1){
                        printf("[Input] 1 - trigger; 0 - cancel: ");
                        std::cin >> i_c;
                        if(i_c == 1){
                            o_camera->call_trigger();
                            i_c = 0;
                        }else{
                            M_RUNNING = 0;
                            break;
                        }
                        sleep(1);
                    }
                }
            }
            pthread_join(t_image_thread, NULL);
            ret = t_camera->close();
            printf("[Debug] Camera close: %d\n", ret);
        }else{
            printf("[Error] failed to open Camera\n");
        }
        em->closeLaser();
        em->disableLightSection(1);
        em->disableLightSection(2);
        em->disableLightSection(3);
        em->disableLightSection(4);
        sleep(1);
        eda::EdaIo::close_io();
        delete t_camera;
    }
    return 0;
}