#!/usr/bin/python3
import time
from libedacamera2 import EdaCamera
from libedaio2 import EdaIo
import sys
import threading
import signal
import cv2
import numpy as np

G_RUNNING = True;

t_width = 0;
t_height = 0;
t_mode = 0;
class Data:
    def __init__(self) -> None:
        self._lock = threading.Lock();
        self._data = None;
        self._len = 0;
    
    def get_data(self):
        self._lock.acquire()
        tData = self._data;
        tLen = self._len;
        self._len = 0;
        self._data = None;
        self._lock.release()
        return (tData, tLen);

    def set_data(self, p_data, p_len):
        self._lock.acquire()
        self._data = p_data;
        self._len = p_len;
        self._lock.release()
            
g_data = Data();

def signal_handler(signal, frame):
    global G_RUNNING;
    print('Caught Ctrl+C / SIGINT signal')
    G_RUNNING = False;

def func_image_data(p_data, p_len):
    # print(f"[Debug] image data: {p_len} - {len(p_data)}!" )
    g_data.set_data(p_data, p_len);
    del p_data;
    return 0;


def thread_data_handle():
    tIndex = 1;
    while(G_RUNNING):
        (tData, tLen) = g_data.get_data()
        if tLen > 0:
            if len(tData) == tLen :
                frame = np.frombuffer(tData, dtype=np.uint8).reshape(t_height,t_width,1)
                frame= cv2.cvtColor(frame, cv2.COLOR_GRAY2RGB)
                cv2.imwrite(f"output_{tIndex}.jpg", frame)
                tIndex += 1;
                del frame;
        del tData;
        time.sleep(1);

def main() -> int:
    global G_RUNNING;
    global t_width;
    global t_height;
    global t_mode;

    signal.signal(signal.SIGINT, signal_handler)

    edaIo = EdaIo.singleton();
    eda = EdaCamera.load_default();
    edaIo.setup();
    edaIo.enableLightSection(1);
    edaIo.enableLightSection(2);
    edaIo.enableLightSection(3);
    edaIo.enableLightSection(4);
    edaIo.openLaser();
    t_mode = input("[Input] work mode[0: CONTINUE, 1: HARDWARE, 4: LEVEL, 5: SOFTWARE]: ")
    t_mode = int(t_mode);
    t_width = input("width: ");
    t_width = int(t_width);
    t_height = input("height: ");
    t_height = int(t_height)
    t_gain = input("gain: ");
    t_exposure = input("exposure: ");
    ret = eda.open(t_mode,t_width, t_height);
    print(f"[Debug] Camera open: {ret}\n");
    if(ret != 0):
        return -1;

    eda.set_gain(int(t_gain));
    eda.set_exposure(int(t_exposure));
    eda.callback_image_ready(func_image_data);
    thread = threading.Thread(target=thread_data_handle);
    thread.start();
    if t_mode == 5 :
        while(True):
            t_c = input("[Input] 1 - trigger; 0 - cancel: ");
            if t_c == "1" :
                eda.call_trigger();
            else:
                G_RUNNING = False;
                break;
            time.sleep(1);
    elif t_mode == 0:
        while G_RUNNING:
            time.sleep(1);
    thread.join();
    eda.close();
    edaIo.disableLightSection(1);
    edaIo.disableLightSection(2);
    edaIo.disableLightSection(3);
    edaIo.disableLightSection(4);
    edaIo.closeLaser();
    return 0;

if __name__ == "__main__":
    sys.exit(main())