#!/usr/bin/python3
import time
from picamera2 import Picamera2, Preview
from libedaio2 import EdaIo,registerInput,registerTrigger,registerTune
import sys

class MyCamera:
    def __init__(self) -> None:
        self.picam2 = Picamera2()
        preview_config = self.picam2.create_preview_configuration()
        self.picam2.configure(preview_config)        
        self._capture = False
        self.index = 1
    
    def start(self) -> None:
        self.picam2.start_preview(Preview.NULL)
        self.picam2.start()
        self._running = True
        
        while(self._running):
            time.sleep(1)
        self.picam2.close()

    def capture(self) -> None:
        if(self._running):
            self.picam2.capture_file(f"{self.index}.jpg")
            print("[Camera] Trigger capture!")
            self.index += 1

    def close(self) -> None:
        self._running = False
        
myCamer = MyCamera()

def func_input(v):
    print("[Debug] Trigger: input!", v)

def func_trigger(v):
    print("[Debug] Trigger: trigger button!", v)

def func_tune(v):
    print("[Debug] Trigger: tune button!", v)
    myCamer.close()

def func_hardware():
    print("[Debug] Trigger: Hardware!")
    myCamer.capture()

def main() -> int:
    eda = EdaIo.singleton()
    registerInput(func_input)
    registerTrigger(func_trigger)
    registerTune(func_tune)
    eda.setup()
    eda.setWorkMode(2)

    myCamer.start()
    return 0

if __name__ == "__main__":
    sys.exit(main())