#!/usr/bin/python3
import time
from picamera2 import Picamera2, Preview
from libedaio2 import EdaIo,registerInput,registerTrigger,registerTune, registerHardware
import sys

class MyCamera:
    def __init__(self) -> None:
        self.eda = EdaIo.singleton()
        self.eda.setup()
        self.eda.setWorkMode(5)
        self.eda.enableLightSection(1)
        self.eda.enableLightSection(2)
        self.eda.enableLightSection(3)
        self.eda.enableLightSection(4)
        self.eda.setLightFlashCount(20)
        self.eda.setLightFlashTime(1)
        
        self.picam2 = Picamera2()
        preview_config = self.picam2.create_preview_configuration()
        self.picam2.configure(preview_config)        
        self._capture = False
        self.index = 1
    
    def start(self) -> None:
        self.picam2.start_preview(Preview.NULL)
        self.picam2.start()
        self._running = True
        
        while(self._running):
            v = input("[capture] liquid:")
            if v == "0":
                break
            self.setLiquid(int(v))
            time.sleep(1)
            self.capture()
        self.picam2.close()

    def capture(self) -> None:
        if(self._running):
            self.eda.callLightFlash()
            self.picam2.capture_file(f"{self.index}.jpg")
            print("[Camera] Trigger capture!")
            self.index += 1

    def close(self) -> None:
        self._running = False
        self.eda.disableLightSection(1)
        self.eda.disableLightSection(2)
        self.eda.disableLightSection(3)
        self.eda.disableLightSection(4)

    def setLiquid(self, v):
        self.eda.setLiquid(v)
        time.sleep(0.1)
        d = self.eda.getLiquid()
        print("[Liquid] py: ",d)
    def testLight(self):
        self.eda.open_light()
        time.sleep(2)
        print("[main] close")
        self.eda.close_light()
        self.eda.open_light()
        time.sleep(4)
        print("[main] close")
        self.eda.close_light()
        self.eda.open_light()
        time.sleep(1)
        print("[main] close")
        self.eda.close_light()
        
myCamer = MyCamera()

def func_input(v):
    print("[Debug] Trigger: input!", v)

def func_trigger(v):
    print("[Debug] Trigger: trigger button!", v)

def func_tune(v):
    print("[Debug] Trigger: tune button!", v)
    myCamer.close()

def func_hardware():
    print("[Debug] Trigger: hardware button!")

def main() -> int:
    registerInput(func_input)
    registerTrigger(func_trigger)
    registerTune(func_tune)    
    # Need work mode: 2
    # registerHardware(func_hardware)
    myCamer.testLight()
    myCamer.start()
    return 0

if __name__ == "__main__":
    sys.exit(main())